#print(__doc__)

# Modified for documentation by Jaques Grobler
# License: BSD 3 clause
import pandas as pd
from pandas_ml import ConfusionMatrix
import numpy as np
import matplotlib.pyplot as plt
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
from sklearn import svm
from sklearn.datasets import make_moons, make_circles, make_classification
from sklearn.neural_network import MLPClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.svm import SVC
from sklearn.gaussian_process import GaussianProcessClassifier
from sklearn.gaussian_process.kernels import RBF
from sklearn.tree import DecisionTreeClassifier
from sklearn.ensemble import RandomForestClassifier, GradientBoostingClassifier,  ExtraTreesClassifier
from sklearn.kernel_approximation import RBFSampler
from sklearn.linear_model import SGDClassifier
from sklearn import preprocessing
from time import time
from sklearn.metrics import accuracy_score
import csv

#Consistently bad:
# "QDA":QuadraticDiscriminantAnalysis()
# "Naive Bayes",GaussianNB(),
# "AdaBoost"   AdaBoostClassifier(),
names = ["Nearest Neighbors BT","Nearest Neighbors KD", "Linear SVM", "RBF SVM", "Gaussian Process",
         "Decision Tree", "Random Forest","GTB", "Neural Net -lbfgs","Neural Net -adam","Neural Net -sgd","Extra trees" ,"SVC","linearSVC","SGDC"]

classifiers = [
    KNeighborsClassifier(5,algorithm="ball_tree"),
    KNeighborsClassifier(5,algorithm="kd_tree"),
    SVC(kernel="linear", C=0.025),
    SVC(gamma=2, C=1),
    GaussianProcessClassifier(1.0 * RBF(1.0), warm_start=True),
    DecisionTreeClassifier(max_depth=5),
    RandomForestClassifier(max_depth=1000, n_estimators=100, max_features=16),
    GradientBoostingClassifier(max_depth=1000, n_estimators=100, max_features=16),
    MLPClassifier(max_iter=100000,solver='lbfgs',tol = 0.0000001),
    MLPClassifier(max_iter=100000, solver='adam',tol = 0.0000001),
    MLPClassifier(max_iter=100000,  solver='sgd',tol = 0.0000001),
    ExtraTreesClassifier(n_estimators=200, max_depth=None, random_state = 0),
    svm.SVC(decision_function_shape='ovo'),
    svm.LinearSVC(C=1.0, class_weight=None, dual=True, fit_intercept=True,
              intercept_scaling=1, loss='squared_hinge', max_iter=100000,
              multi_class='ovr', penalty='l2', random_state=None, tol=0.0001,
              verbose=0),
    SGDClassifier(alpha=0.0001, average=False, class_weight=None, epsilon=0.1,
       eta0=0.0, fit_intercept=True, l1_ratio=0.15,
       learning_rate='optimal', loss='hinge', n_iter=5, n_jobs=1,
       penalty='l2', power_t=0.5, random_state=None, shuffle=True,
       verbose=0, warm_start=False)

]



dataset = 'C:\Users\osmun\Desktop\Work\CHAIR\Master\Datasets\BOZO'

train_dataset = pd.read_csv('{}.csv'.format(dataset))
train_dataset=train_dataset[train_dataset.Label!='Core Active']
test_dataset = pd.read_csv('{}_test.csv'.format(dataset))
test_dataset = test_dataset[test_dataset.Label!='Core Active']
train_dataset = train_dataset.drop(['year', 'month', 'Date', 'time'], 1)
test_dataset = test_dataset.drop(['year', 'month', 'Date', 'time'], 1)

#train_dataset = preprocessing.MinMaxScaler().fit(train_dataset[['Front Left+',	'Front Left'	,'Front Right',
 #                                                             'Front Right+',	'Mid Left+',	'Mid Left',
  #                                                            'Mid Right',	'Mid Rght+',	'Back Left',	'Back Right',
   #                                                          'Back+ Left',	'Back+ Right',	'Top Left',	'Top Right',
    #                                                         'Lumbar Left',	'Lumbar Right']])
#test_dataset = preprocessing.MinMaxScaler().fit(test_dataset[['Front Left+',	'Front Left'	,'Front Right',
 #                                                             'Front Right+',	'Mid Left+',	'Mid Left',
  #                                                            'Mid Right',	'Mid Rght+',	'Back Left',	'Back Right',
   #                                                          'Back+ Left',	'Back+ Right',	'Top Left',	'Top Right',
    #                                                         'Lumbar Left',	'Lumbar Right']])
train, validation = train_test_split(train_dataset, test_size=0.1)

train_labels = train.Label
train_features = train.drop('Label', 1)
#train_features =(train_features - train_features.mean()) / (train_features.max() - train_features.min())

validation_labels = validation.Label
validation_features = validation.drop('Label', 1)
#validation_features =(validation_features - validation_features.mean()) / (validation_features.max() - validation_features.min())

test_labels = test_dataset.Label
test_features = test_dataset.drop('Label', 1)
#test_features =(test_features - test_features.mean()) / (test_features.max() - test_features.min())
classifier_dict = {}
i = 0
for name in names:
    classifier_dict[name] = classifiers[i]
    i = i + 1
validation_results = {}
test_results = {}

evalDict={}
test_labels2 = test_labels.reset_index(drop=True)
for name in names:
    #tStart= time()
    classifier_dict[name].fit(train_features, train_labels)
    #tTrain = time()
    validation_results[name] = classifier_dict[name].predict(validation_features)
    test_results[name] = classifier_dict[name].predict(test_features)
    #tEnd= time()
    #print 'Total time {0}\t: {1}\t, of which training was:{2}\t, and the predictions was: {3}'.format(name,tEnd-tStart,tTrain-tStart,tEnd-tTrain),'\n'
    evalDict[name] = accuracy_score(test_labels2,test_results[name])
    #print name, evalDict[name]







figureDict = {}

#i = 0
i = 1
VL=validation_labels.tolist()

if i == 1:
    for name in names:
        cmValidation = ConfusionMatrix(VL,validation_results[name])
        cmTest = ConfusionMatrix(test_labels2, test_results[name])
        figureDict[i] = plt.figure('{0}, Test'.format(name))

        figureDict[i]
        figureDict[i].add_subplot(1, 2, 1)
        cmTest.plot(backend='seaborn', normalized=True)
        plt.xticks(rotation=45)
        plt.yticks(rotation='horizontal')
        plt.title('{0}, Test'.format(name))

        figureDict[i].add_subplot(1, 2, 2)
        cmValidation.plot(backend='seaborn', normalized=True)
        plt.xticks(rotation=45)
        plt.yticks(rotation='horizontal')
        plt.title('{0}, Validation'.format(name))

        i += 1
combined_prediction = pd.DataFrame(
    {'Extra Trees': test_results['Extra trees'],
     'NNKD': test_results['Nearest Neighbors KD'],
     'Linear SVC': test_results['linearSVC']
    })
#print combined_prediction.iloc[0][0]
combined= []
#print combined_prediction
for i in range (0,len(combined_prediction.index)):
    #print combined_prediction.iloc[i]
    combined.append( combined_prediction.iloc[i].value_counts().index[0])


evalDict['combined'] =  accuracy_score(test_labels,combined)
Sorted_Dict_Values = sorted(evalDict.values(), reverse=True)
Sorted_Dict_Keys = sorted(evalDict, key=evalDict.get, reverse=True)
for i in range(len(evalDict)):
    print'Accuracy {0} : {1}'.format(Sorted_Dict_Keys[i],Sorted_Dict_Values[i])

evalFig = plt.figure('Accuracy Scores')


plt.bar(range(len(evalDict)),Sorted_Dict_Values,align='center')
plt.xticks(range(len(evalDict)),Sorted_Dict_Keys,rotation=45)



myfile= open('{}machineLearningTestNewCLFs.csv'.format(dataset),'ab')
wr = csv.writer(myfile, quoting=csv.QUOTE_ALL)

#If making dataset, uncomment following
#wr.writerow(names)

accuracies = []
for name in names:
    accuracies.append(evalDict[name])

wr.writerow(accuracies)
myfile.close()
plt.show()
